version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries, loadglob(../external/lib/input_parameters/input_param.txt)

program main

    local outcome "Lnhcccom"

    load_data_main_model, ///
        mover_dataset(../external_links/compiled/CompiledPatYrDataMovers) ///
        nonmover_dataset(../external_links/compiled/CompiledPatYrDataNonMovers) ///
        variables(bene_id year hrrnum hrr_orig hrr_dest `outcome' *_samp mover) ///
        rhs(../external_links/rhs_matrix/rhs_matrix_noadj.dta) saving(../temp/data_loaded) 
    keep if ${`outcome'_samp}==1 & rel_yr_rit!=0
    gen fwgt = ${mover_weight}
    replace fwgt = ${nonmover_weight} if mover==0
    drop _Ihrrnum_1

    cap mkdir ../output/`outcome'/
    datasignature set, saving(../output/`outcome'/datasignature, replace) reset
    
    areg `outcome' _I* _Y* _RY* _A* [aw=fwgt], a(gbene_id) cl(gbene_id)
    matrix b = e(b)
    matrix V = e(V)

    reg `outcome' _I*
    matrix a = e(b)

    unab vars: _I*
    local k = wordcount("`vars'")
    matrix T = J(1,`k',-1/(`k'+1)) \ (I(`k') - J(`k',`k',1/(`k'+1)))
    matrix b = T*b[1,1..`k']'
    matrix V = T*V[1..`k',1..`k']*T'
    matrix a = T*a[1,1..`k']'
    matrix test = cholesky(V)

    gen hcccom = exp(Lnhcccom)
    collapse (mean) hcccom [aw=fwgt], by(hrrnum)
    save ../output/`outcome'/avgs, replace
    clear
    set obs 1
    gen vars = "_Ihrrnum_1 `vars'"
    save ../output/`outcome'/vars, replace
    clear
    svmat b
    save ../output/`outcome'/beta, replace    
    clear
    svmat V
    save ../output/`outcome'/VCE, replace
    clear
    svmat a
    save ../output/`outcome'/alpha, replace

end

* Execute
main

