version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries, loadglob(../external/lib/input_parameters/input_param.txt)

mata
    function fgls(y,x,v,w,s) {

      gamma = invsym(x'*invsym(w)*x)*(x'*invsym(w)*y)
      epsil = y-x*gamma
          N = length(x)
          K = length(gamma)
      g_vce = (N/(N-K))*invsym(x'*x)*x'*(diag(epsil:*epsil)+v)*x*invsym(x'*x)

      sigm2 = 0
         wt = 0 
      for (i=1;i<=N;i++) {
        sigm2 = sigm2+(1/(s+v[i,i]))*((N/(N-K))*epsil[i,1]^2-v[i,i])
           wt = wt+(1/(s+v[i,i]))
      }
      sigm2 = sigm2/wt
      s_vce = 0
      for (i=1;i<=N;i++) {
        s_vce = s_vce+(1/((s+v[i,i])*wt))*(epsil[i,1]^2-v[i,i]-sigm2)^2
      }
      s_vce = s_vce / (N-K)

      st_matrix("r(gamma)",gamma)
      st_matrix("r(g_vce)",g_vce)
      st_matrix("r(sigm2)",sigm2)
      st_matrix("r(s_vce)",s_vce)

    }
end

program estimate

    local model "risk scores"
    local outcome "Lnhcccom"

    cap mkdir ../output/`outcome'/
    use "../../Model (`model')/output/`outcome'/alpha", clear
    mata X = st_data(.,"a")
    use "../../Model (`model')/output/`outcome'/beta", clear
    mkmat *, mat(b)
    mata Y = st_data(.,"b")
    use "../../Model (`model')/output/`outcome'/VCE", clear
    mata V = st_data(.,"*")

    mata W_0 = I(length(X))
    mata fgls(Y,X,V,W_0,0)
    matrix results_0 = r(gamma)\r(g_vce)\r(sigm2)\r(s_vce)
    forval i = 1/10 {
      local sigm2 = results_`=`i'-1'[3,1]
      mata W_`i' = `sigm2'*I(length(X))+V
      mata fgls(Y,X,V,W_`i',`sigm2')
      matrix results_`i' = r(gamma)\r(g_vce)\r(sigm2)\r(s_vce)
    }  
    matrix results = results_0, results_1, results_10
    forval i = 1/3 {
      matrix results[2,`i'] = sqrt(results[2,`i'])
      matrix results[3,`i'] = sqrt(results[3,`i'])
      matrix results[4,`i'] = sqrt(results[4,`i'])*0.5*results[3,`i']^-1
    }
    local gamma = results_10[1,1]
    local sigm2 = results_10[3,1]
    mata B = `sigm2'*I(length(X))*invsym(`sigm2'*I(length(X))+V)
    mata P = B*Y+(I(length(X))-B)*`gamma'*X

    clear
    svmat results
    outsheet using ../output/`outcome'/HLM_ests.csv, replace
    clear
    use "../../Model (`model')/output/`outcome'/vars", clear 
    split var
    drop vars

    gen i = 1
    reshape long vars, i(i)
    drop i _j
    replace vars = substr(vars,10,.)
    destring vars, replace
    rename vars hrrnum
    gen post = .
    mata: st_store(.,"post",P)
    gen beta = .
    mata: st_store(.,"beta",Y)
    gen alpha = .
    mata: st_store(.,"alpha",X) 
    save ../output/`outcome'/posts, replace

end

* Execute
estimate

